@@ -12,6 +12,7 @@ __all__ = [
 
 import copy
 import datetime as dt
+import inspect
 import re
 import shlex
 import shutil
@@ -80,6 +81,44 @@ class Vcs(Enum):
     Fossil = "fossil"
 
 
+class Pattern(Enum):
+    Default = "default"
+    DefaultUnprefixed = "default-unprefixed"
+
+    def regex(self) -> str:
+        """
+        Get the regular expression for this preset pattern.
+        """
+        variants = {
+            Pattern.Default: VERSION_SOURCE_PATTERN,
+            Pattern.DefaultUnprefixed: VERSION_SOURCE_PATTERN.replace("^v", "^v?", 1),
+        }
+        return variants[self]
+
+    @staticmethod
+    def parse(pattern: Union[str, "Pattern"]) -> str:
+        """
+        Parse a pattern string into a regular expression.
+
+        If the pattern contains the capture group `?P<base>`, then it is
+        returned as-is. Otherwise, it is interpreted as a variant of the
+        `Pattern` enum.
+        """
+        if isinstance(pattern, str) and "?P<base>" in pattern:
+            return pattern
+
+        try:
+            pattern = Pattern(pattern)
+        except ValueError:
+            raise ValueError(
+                (
+                    "The pattern '{}' does not contain the capture group '?P<base>'"
+                    " and is not a known preset like '{}'"
+                ).format(pattern, Pattern.Default.value)
+            )
+        return pattern.regex()
+
+
 def _run_cmd(
     command: str,
     codes: Sequence[int] = (0,),
@@ -119,7 +158,7 @@ _MatchedVersionPattern = NamedTuple(
 
 
 def _match_version_pattern(
-    pattern: str, sources: Sequence[str], latest_source: bool
+    pattern: Union[str, Pattern], sources: Sequence[str], latest_source: bool
 ) -> _MatchedVersionPattern:
     """
     :return: Tuple of:
@@ -138,8 +177,19 @@ def _match_version_pattern(
     tagged_metadata = None
     epoch = None  # type: Optional[Union[str, int]]
 
+    pattern = Pattern.parse(pattern)
+
     for source in sources[:1] if latest_source else sources:
-        pattern_match = re.search(pattern, source)
+        try:
+            pattern_match = re.search(pattern, source)
+        except re.error as e:
+            raise re.error(
+                "Pattern '{}' is an invalid regular expression: {}".format(
+                    pattern, e.msg  # type: ignore
+                ),
+                e.pattern,  # type: ignore
+                e.pos,  # type: ignore
+            )
         if pattern_match is None:
             newer_unmatched_tags.append(source)
             continue
@@ -292,9 +342,10 @@ class _GitRefInfo:
             return "refs/tags/{}".format(ref)
 
     @staticmethod
-    def from_git_tag_topo_order() -> Mapping[str, int]:
+    def from_git_tag_topo_order(tag_branch: str) -> Mapping[str, int]:
         code, logmsg = _run_cmd(
-            'git log --simplify-by-decoration --topo-order --decorate=full HEAD "--format=%H%d"'
+            "git log --simplify-by-decoration --topo-order --decorate=full"
+            ' {} "--format=%H%d"'.format(tag_branch)
         )
         tag_lookup = {}
 
@@ -519,19 +570,24 @@ class Version:
                 new_version.revision = revision
                 out = format(new_version)
             else:
-                out = format.format(
-                    base=base,
-                    stage=_blank(self.stage, ""),
-                    revision=_blank(revision, ""),
-                    distance=_blank(self.distance, ""),
-                    commit=_blank(self.commit, ""),
-                    tagged_metadata=_blank(self.tagged_metadata, ""),
-                    dirty="dirty" if self.dirty else "clean",
-                    epoch=_blank(self.epoch, ""),
-                    branch=_blank(self.branch, ""),
-                    branch_escaped=_escape_branch(_blank(self.branch, "")),
-                    timestamp=self.timestamp.strftime("%Y%m%d%H%M%S") if self.timestamp else "",
-                )
+                try:
+                    out = format.format(
+                        base=base,
+                        stage=_blank(self.stage, ""),
+                        revision=_blank(revision, ""),
+                        distance=_blank(self.distance, ""),
+                        commit=_blank(self.commit, ""),
+                        tagged_metadata=_blank(self.tagged_metadata, ""),
+                        dirty="dirty" if self.dirty else "clean",
+                        epoch=_blank(self.epoch, ""),
+                        branch=_blank(self.branch, ""),
+                        branch_escaped=_escape_branch(_blank(self.branch, "")),
+                        timestamp=self.timestamp.strftime("%Y%m%d%H%M%S") if self.timestamp else "",
+                    )
+                except KeyError as e:
+                    raise KeyError("Format contains invalid placeholder: {}".format(e))
+                except ValueError as e:
+                    raise ValueError("Format is invalid: {}".format(e))
             if style is not None:
                 check_version(out, style)
             return out
@@ -601,7 +657,7 @@ class Version:
         return out
 
     @classmethod
-    def parse(cls, version: str, pattern: str = VERSION_SOURCE_PATTERN) -> "Version":
+    def parse(cls, version: str, pattern: Union[str, Pattern] = Pattern.Default) -> "Version":
         """
         Attempt to parse a string into a Version instance.
 
@@ -613,8 +669,10 @@ class Version:
             Refer to `from_any_vcs` for more info.
         """
         try:
-            prefixed = version if version.startswith("v") else "v{}".format(version)
-            matched_pattern = _match_version_pattern(pattern, [prefixed], True)
+            normalized = version
+            if not version.startswith("v") and pattern in [VERSION_SOURCE_PATTERN, Pattern.Default]:
+                normalized = "v{}".format(version)
+            matched_pattern = _match_version_pattern(pattern, [normalized], True)
         except ValueError:
             return cls(version)
 
@@ -698,7 +756,13 @@ class Version:
         return bumped
 
     @classmethod
-    def from_git(cls, pattern: str = VERSION_SOURCE_PATTERN, latest_tag: bool = False) -> "Version":
+    def from_git(
+        cls,
+        pattern: Union[str, Pattern] = Pattern.Default,
+        latest_tag: bool = False,
+        tag_branch: Optional[str] = None,
+        full_commit: bool = False,
+    ) -> "Version":
         r"""
         Determine a version based on Git tags.
 
@@ -707,8 +771,13 @@ class Version:
         :param latest_tag: If true, only inspect the latest tag on the latest
             tagged commit for a pattern match. If false, keep looking at tags
             until there is a match.
+        :param tag_branch: Branch on which to find tags, if different than the
+            current branch.
+        :param full_commit: Get the full commit hash instead of the short form.
         """
         _detect_vcs(Vcs.Git)
+        if tag_branch is None:
+            tag_branch = "HEAD"
 
         code, msg = _run_cmd("git symbolic-ref --short HEAD", codes=[0, 128])
         if code == 128:
@@ -716,7 +785,10 @@ class Version:
         else:
             branch = msg
 
-        code, msg = _run_cmd('git log -n 1 --format="format:%h"', codes=[0, 128])
+        code, msg = _run_cmd(
+            'git log -n 1 --format="format:{}"'.format("%H" if full_commit else "%h"),
+            codes=[0, 128],
+        )
         if code == 128:
             return cls("0.0.0", distance=0, dirty=True, branch=branch)
         commit = msg
@@ -733,8 +805,8 @@ class Version:
                 dirty = True
 
         code, msg = _run_cmd(
-            'git for-each-ref "refs/tags/**" --merged HEAD'
-            ' --format "%(refname)'
+            'git for-each-ref "refs/tags/**" --merged {}'.format(tag_branch)
+            + ' --format "%(refname)'
             "@{%(objectname)"
             "@{%(creatordate:iso-strict)"
             "@{%(*committerdate:iso-strict)"
@@ -757,7 +829,7 @@ class Version:
             )
 
         detailed_tags = []  # type: List[_GitRefInfo]
-        tag_topo_lookup = _GitRefInfo.from_git_tag_topo_order()
+        tag_topo_lookup = _GitRefInfo.from_git_tag_topo_order(tag_branch)
 
         for line in msg.strip().splitlines():
             parts = line.split("@{")
@@ -788,7 +860,10 @@ class Version:
 
     @classmethod
     def from_mercurial(
-        cls, pattern: str = VERSION_SOURCE_PATTERN, latest_tag: bool = False
+        cls,
+        pattern: Union[str, Pattern] = Pattern.Default,
+        latest_tag: bool = False,
+        full_commit: bool = False,
     ) -> "Version":
         r"""
         Determine a version based on Mercurial tags.
@@ -798,6 +873,7 @@ class Version:
         :param latest_tag: If true, only inspect the latest tag on the latest
             tagged commit for a pattern match. If false, keep looking at tags
             until there is a match.
+        :param full_commit: Get the full commit hash instead of the short form.
         """
         _detect_vcs(Vcs.Mercurial)
 
@@ -807,7 +883,9 @@ class Version:
         code, msg = _run_cmd("hg branch")
         branch = msg
 
-        code, msg = _run_cmd('hg id --template "{id|short}"')
+        code, msg = _run_cmd(
+            'hg id --template "{}"'.format("{id}" if full_commit else "{id|short}")
+        )
         commit = msg if set(msg) != {"0"} else None
 
         code, msg = _run_cmd('hg log --limit 1 --template "{date|rfc3339date}"')
@@ -858,7 +936,7 @@ class Version:
 
     @classmethod
     def from_darcs(
-        cls, pattern: str = VERSION_SOURCE_PATTERN, latest_tag: bool = False
+        cls, pattern: Union[str, Pattern] = Pattern.Default, latest_tag: bool = False
     ) -> "Version":
         r"""
         Determine a version based on Darcs tags.
@@ -916,7 +994,10 @@ class Version:
 
     @classmethod
     def from_subversion(
-        cls, pattern: str = VERSION_SOURCE_PATTERN, latest_tag: bool = False, tag_dir: str = "tags"
+        cls,
+        pattern: Union[str, Pattern] = Pattern.Default,
+        latest_tag: bool = False,
+        tag_dir: str = "tags",
     ) -> "Version":
         r"""
         Determine a version based on Subversion tags.
@@ -994,7 +1075,7 @@ class Version:
 
     @classmethod
     def from_bazaar(
-        cls, pattern: str = VERSION_SOURCE_PATTERN, latest_tag: bool = False
+        cls, pattern: Union[str, Pattern] = Pattern.Default, latest_tag: bool = False
     ) -> "Version":
         r"""
         Determine a version based on Bazaar tags.
@@ -1070,7 +1151,7 @@ class Version:
 
     @classmethod
     def from_fossil(
-        cls, pattern: str = VERSION_SOURCE_PATTERN, latest_tag: bool = False
+        cls, pattern: Union[str, Pattern] = Pattern.Default, latest_tag: bool = False
     ) -> "Version":
         r"""
         Determine a version based on Fossil tags.
@@ -1174,7 +1255,12 @@ class Version:
 
     @classmethod
     def from_any_vcs(
-        cls, pattern: str = VERSION_SOURCE_PATTERN, latest_tag: bool = False, tag_dir: str = "tags"
+        cls,
+        pattern: Union[str, Pattern] = Pattern.Default,
+        latest_tag: bool = False,
+        tag_dir: str = "tags",
+        tag_branch: Optional[str] = None,
+        full_commit: bool = False,
     ) -> "Version":
         r"""
         Determine a version based on a detected version control system.
@@ -1187,22 +1273,32 @@ class Version:
             It may also contain a group named `tagged_metadata` corresponding to extra
             metadata after the main part of the version (typically after a plus sign).
             There may also be a group named `epoch` for the PEP 440 concept.
+
+            If the `base` group is not included, then this will be interpreted
+            as the name of a variant of the `Pattern` enum. For example, passing
+            `"default"` is the same as passing `Pattern.Default`.
         :param latest_tag: If true, only inspect the latest tag on the latest
             tagged commit for a pattern match. If false, keep looking at tags
             until there is a match.
         :param tag_dir: Location of tags relative to the root.
             This is only used for Subversion.
+        :param tag_branch: Branch on which to find tags, if different than the
+            current branch. This is only used for Git currently.
+        :param full_commit: Get the full commit hash instead of the short form.
+            This is only used for Git and Mercurial.
         """
         vcs = _detect_vcs()
-        return cls._do_vcs_callback(vcs, pattern, latest_tag, tag_dir)
+        return cls._do_vcs_callback(vcs, pattern, latest_tag, tag_dir, tag_branch, full_commit)
 
     @classmethod
     def from_vcs(
         cls,
         vcs: Vcs,
-        pattern: str = VERSION_SOURCE_PATTERN,
+        pattern: Union[str, Pattern] = Pattern.Default,
         latest_tag: bool = False,
         tag_dir: str = "tags",
+        tag_branch: Optional[str] = None,
+        full_commit: bool = False,
     ) -> "Version":
         r"""
         Determine a version based on a specific VCS setting.
@@ -1218,11 +1314,23 @@ class Version:
             until there is a match.
         :param tag_dir: Location of tags relative to the root.
             This is only used for Subversion.
+        :param tag_branch: Branch on which to find tags, if different than the
+            current branch. This is only used for Git currently.
+        :param full_commit: Get the full commit hash instead of the short form.
+            This is only used for Git and Mercurial.
         """
-        return cls._do_vcs_callback(vcs, pattern, latest_tag, tag_dir)
+        return cls._do_vcs_callback(vcs, pattern, latest_tag, tag_dir, tag_branch, full_commit)
 
     @classmethod
-    def _do_vcs_callback(cls, vcs: Vcs, pattern: str, latest_tag: bool, tag_dir: str) -> "Version":
+    def _do_vcs_callback(
+        cls,
+        vcs: Vcs,
+        pattern: Union[str, Pattern],
+        latest_tag: bool,
+        tag_dir: str,
+        tag_branch: Optional[str],
+        full_commit: bool,
+    ) -> "Version":
         mapping = {
             Vcs.Any: cls.from_any_vcs,
             Vcs.Git: cls.from_git,
@@ -1232,10 +1340,16 @@ class Version:
             Vcs.Bazaar: cls.from_bazaar,
             Vcs.Fossil: cls.from_fossil,
         }  # type: Mapping[Vcs, Callable[..., "Version"]]
-        kwargs = {"pattern": pattern, "latest_tag": latest_tag}
-        if vcs == Vcs.Subversion:
-            kwargs["tag_dir"] = tag_dir
-        return mapping[vcs](**kwargs)
+        kwargs = {"pattern": pattern, "latest_tag": latest_tag}  # type: dict
+        callback = mapping[vcs]
+        for kwarg, value in [
+            ("tag_branch", tag_branch),
+            ("full_commit", full_commit),
+            ("tag_dir", tag_dir),
+        ]:
+            if kwarg in inspect.getfullargspec(callback).args:
+                kwargs[kwarg] = value
+        return callback(**kwargs)
 
 
 def check_version(version: str, style: Style = Style.Pep440) -> None:
