@@ -2,7 +2,7 @@ import argparse
 import sys
 from typing import Mapping, Optional
 
-from dunamai import check_version, Version, Style, Vcs, VERSION_SOURCE_PATTERN
+from dunamai import check_version, Version, Pattern, Style, Vcs, VERSION_SOURCE_PATTERN
 
 
 common_sub_args = [
@@ -45,7 +45,9 @@ common_sub_args = [
             " It may also contain a group named `tagged_metadata` corresponding to extra"
             " metadata after the main part of the version (typically after a plus sign)."
             " There may also be a group named `epoch` for the PEP 440 concept."
-        ),
+            " If the `base` group is not present, then instead this will be interpreted"
+            " as a named preset, which may be one of the following: {}"
+        ).format(", ".join(["`{}`".format(x.value) for x in Pattern])),
     },
     {
         "triggers": ["--format"],
@@ -91,6 +93,25 @@ common_sub_args = [
             " Does nothing when on a commit with a version tag."
         ),
     },
+    {
+        "vcs": [Vcs.Git, Vcs.Mercurial],
+        "triggers": ["--full-commit"],
+        "action": "store_true",
+        "dest": "full_commit",
+        "default": False,
+        "help": "Get the full commit hash instead of the short form",
+    },
+    {
+        "vcs": [Vcs.Git],
+        "triggers": ["--tag-branch"],
+        "help": "Branch on which to find tags, if different than the current branch",
+    },
+    {
+        "vcs": [Vcs.Subversion],
+        "triggers": ["--tag-dir"],
+        "default": "tags",
+        "help": "Location of tags relative to the root",
+    },
 ]
 cli_spec = {
     "description": "Generate dynamic versions",
@@ -102,14 +123,7 @@ cli_spec = {
             "sub": {
                 Vcs.Any.value: {
                     "description": "Generate version from any detected VCS",
-                    "args": [
-                        *common_sub_args,
-                        {
-                            "triggers": ["--tag-dir"],
-                            "default": "tags",
-                            "help": "Location of tags relative to the root (Subversion)",
-                        },
-                    ],
+                    "args": common_sub_args,
                 },
                 Vcs.Git.value: {
                     "description": "Generate version from Git",
@@ -125,14 +139,7 @@ cli_spec = {
                 },
                 Vcs.Subversion.value: {
                     "description": "Generate version from Subversion",
-                    "args": [
-                        *common_sub_args,
-                        {
-                            "triggers": ["--tag-dir"],
-                            "default": "tags",
-                            "help": "Location of tags relative to the root",
-                        },
-                    ],
+                    "args": common_sub_args,
                 },
                 Vcs.Bazaar.value: {
                     "description": "Generate version from Bazaar",
@@ -166,7 +173,7 @@ cli_spec = {
 
 
 def build_parser(
-    spec: Mapping, parser: Optional[argparse.ArgumentParser] = None
+    spec: Mapping, parser: Optional[argparse.ArgumentParser] = None, vcs: Optional[Vcs] = None
 ) -> argparse.ArgumentParser:
     if parser is None:
         parser = argparse.ArgumentParser(
@@ -174,8 +181,17 @@ def build_parser(
         )
     if "args" in spec:
         for arg in spec["args"]:
+            help = arg["help"]
+            if "vcs" in arg:
+                if vcs not in [*arg["vcs"], Vcs.Any]:
+                    continue
+                help += " (only: {})".format(", ".join([x.name for x in arg["vcs"]]))
             triggers = arg["triggers"]
-            parser.add_argument(*triggers, **{k: v for k, v in arg.items() if k != "triggers"})
+            parser.add_argument(
+                *triggers,
+                help=help,
+                **{k: v for k, v in arg.items() if k not in ["triggers", "help", "vcs"]}
+            )
     if "sub" in spec:
         subparsers = parser.add_subparsers(dest=spec["sub_dest"])
         subparsers.required = True
@@ -186,7 +202,7 @@ def build_parser(
                 help=sub_spec.get("description"),
                 formatter_class=argparse.ArgumentDefaultsHelpFormatter,
             )
-            build_parser(sub_spec, subparser)
+            build_parser(sub_spec, subparser, Vcs(name) if spec["sub_dest"] == "vcs" else None)
 
     return parser
 
@@ -217,8 +233,10 @@ def from_vcs(
     debug: bool,
     bump: bool,
     tagged_metadata: bool,
+    tag_branch: Optional[str],
+    full_commit: bool,
 ) -> None:
-    version = Version.from_vcs(vcs, pattern, latest_tag, tag_dir)
+    version = Version.from_vcs(vcs, pattern, latest_tag, tag_dir, tag_branch, full_commit)
     print(version.serialize(metadata, dirty, format, style, bump, tagged_metadata=tagged_metadata))
     if debug:
         print("# Matched tag: {}".format(version._matched_tag), file=sys.stderr)
@@ -230,6 +248,8 @@ def main() -> None:
     try:
         if args.command == "from":
             tag_dir = getattr(args, "tag_dir", "tags")
+            tag_branch = getattr(args, "tag_branch", None)
+            full_commit = getattr(args, "full_commit", False)
             from_vcs(
                 Vcs(args.vcs),
                 args.pattern,
@@ -242,6 +262,8 @@ def main() -> None:
                 args.debug,
                 args.bump,
                 args.tagged_metadata,
+                tag_branch,
+                full_commit,
             )
         elif args.command == "check":
             version = from_stdin(args.version)
